#include <math.h>
#define float double

#include "nrutil.c"

#define EPS 1.0e-6
#define JMAX 20
#define JMAXP (JMAX+1)
#define K 5
#define FUNC(x) ((*func)(x))

/*
Here EPS is the fractional accuracy desired, as determined by the extrapolation error estimate;
JMAX limits the total number of steps; K is the number of points used in the extrapolation.
*/


float trapzd(float (*func)(float), float a, float b, int n)
/* 
This routine computes the nth stage of refinement of an extended trapezoidal rule. func is input
as a pointer to the function to be integrated between limits a and b, also input. When called with
n=1, the routine returns the crudest estimate of \int_a^b f(x)dx. Subsequent calls with n=2,3,...
(in that sequential order) will improve the accuracy by adding 2^(n-2) additional interior points.
*/
{
	float x,tnm,sum,del;
	static float s;
	int it,j;

	if (n == 1) {
		return (s=0.5*(b-a)*(FUNC(a)+FUNC(b)));
	} 
	else {
		for (it=1,j=1;j<n-1;j++) it <<= 1;
		tnm=it;
		del=(b-a)/tnm; 				// This is the spacing of the points to be added.
		x=a+0.5*del;
		for (sum=0.0,j=1;j<=it;j++,x+=del) sum += FUNC(x);
		s=0.5*(s+(b-a)*sum/tnm);  		// This replaces s by its refined value.
	
		printf("lepeskoz: %.12f,\t trapez: %g \n", 0.5*del, s);
		
		return s;
	}
}



void polint(float xa[], float ya[], int n, float x, float *y, float *dy)
/*
Given arrays xa[1..n] and ya[1..n], and given a value x, this routine returns a value y, and
an error estimate dy. If P(x) is the polynomial of degree N - 1 such that P(xai) = yai, i =
1, . . . , n, then the returned value y = P(x).
*/
{
	int i,m,ns=1;
	float den,dif,dift,ho,hp,w;
	float *c,*d;

	dif=fabs(x-xa[1]);
	c=vector(1,n);
	d=vector(1,n);
	for (i=1;i<=n;i++) {					// Here we find the index ns of the closest table entry,
		if ( (dift=fabs(x-xa[i])) < dif) {
			ns=i;
			dif=dift;
		}
		c[i]=ya[i];							// and initialize the tableau of cs and ds.
		d[i]=ya[i];
	}

	*y=ya[ns--];							// This is the initial approximation to y.
	for (m=1;m<n;m++) {						// For each column of the tableau,
		for (i=1;i<=n-m;i++) {				// we loop over the current cs and ds and update them. 
			ho=xa[i]-x;
			hp=xa[i+m]-x;
			w=c[i+1]-d[i];
			if ( (den=ho-hp) == 0.0) nrerror("Error in routine polint");
					// This error can occur only if two input xas are (to within roundoff) identical.
			den=w/den;
			d[i]=hp*den;						// Here the cs and ds are updated.
			c[i]=ho*den;
	
		}
		*y += (*dy=(2*ns < (n-m) ? c[ns+1] : d[ns--]));
		/*
		After each column in the tableau is completed, we decide which correction, c or d,
		we want to add to our accumulating value of y, i.e., which path to take through the
		tableauforking up or down. We do this in such a way as to take the most straight
		line route through the tableau to its apex, updating ns accordingly to keep track of
		where we are. This route keeps the partial approximations centered (insofar as possible)
		on the target x. The last dy added is thus the error indication.
		*/
	}
	free_vector(d,1,n);
	free_vector(c,1,n);
}

float qromb(float (*func)(float), float a, float b)
/*
Returns the integral of the function func from a to b. Integration is performed by Rombergs
method of order 2K, where, e.g., K=2 is Simpsons rule.
*/
{
	void polint(float xa[], float ya[], int n, float x, float *y, float *dy);
	float trapzd(float (*func)(float), float a, float b, int n);
	void nrerror(char error_text[]);
	float ss,dss;
	float s[JMAXP],h[JMAXP+1]; // These store the successive trapezoidal approximations and their relative stepsizes. 
	int j;

	h[1]=1.0;
	for (j=1;j<=JMAX;j++) {
		s[j]=trapzd(func,a,b,j);
		if (j >= K) {
			polint(&h[j-K],&s[j-K],K,0.0,&ss,&dss);
			if (fabs(dss) <= EPS*fabs(ss)) return ss;
		}
		h[j+1]=0.25*h[j];
		/*
		This is a key step: The factor is 0.25 even though the stepsize is decreased by only
		0.5. This makes the extrapolation a polynomial in h2 as allowed by equation (4.2.1),
		not just a polynomial in h.
		*/
	}

	nrerror("Too many steps in routine qromb");

	return 0.0;			// Never get here.
}


void main (int argc, char *argv[])
{

	int i;
	float a = 0.0, b = 3.1416, s = 0.0;

	for (i=1; i<argc; i++ ){
		
		switch(i){
	    	case 1: 
			a = atof(argv[1]); 
			break;
		case 2:
			b = atof(argv[2]); 
			break;
		} 
	}

	if    ( a < b ) s = qromb( cos, a, b);
	else if ( b < a ) s = - qromb( cos, b, a);
	
	printf("\n\n\n\t %g\n", b);
	printf("\t /\n");
	printf("\t | cos (x) dx = %.16f \t (Romberg)\n", s);
	printf("\t /\n");
	printf("\t %g\n", a);

}
