#include <math.h>
#include <stdlib.h>
#include "nrutil.c"
#define EPS 1.0e-6
#define JMAX 20
#define FUNC(x) ((*func)(x))
#define float double



float trapzd(float (*func)(float), float a, float b, int n)
/* 
This routine computes the nth stage of refinement of an extended trapezoidal rule. func is input
as a pointer to the function to be integrated between limits a and b, also input. When called with
n=1, the routine returns the crudest estimate of \int_a^b f(x)dx. Subsequent calls with n=2,3,...
(in that sequential order) will improve the accuracy by adding 2^(n-2) additional interior points.
*/
{
	float x,tnm,sum,del;
	static float s;
	int it,j;

	if (n == 1) {
		return (s=0.5*(b-a)*(FUNC(a)+FUNC(b)));
	} 
	else {
		for (it=1,j=1;j<n-1;j++) it <<= 1;
		tnm=it;
		del=(b-a)/tnm; 				// This is the spacing of the points to be added.
		x=a+0.5*del;
		for (sum=0.0,j=1;j<=it;j++,x+=del) sum += FUNC(x);
		s=0.5*(s+(b-a)*sum/tnm);  		// This replaces s by its refined value.
	
		printf("lepeskoz: %.12f,\t trapez: %g ", 0.5*del, s);
		
		return s;
	}
}


float qsimp(float (*func)(float), float a, float b)
/*
Returns the integral of the function func from a to b. The parameters EPS can be set to the
desired fractional accuracy and JMAX so that 2 to the power JMAX-1 is the maximum allowed
number of steps. Integration is performed by Simpsons rule.
*/
{
	float trapzd(float (*func)(float), float a, float b, int n);
	void nrerror(char error_text[]);
	int j;
	float s,st,ost=0.0,os=0.0;
	
	for (j=1;j<=JMAX;j++) {
		st=trapzd(func,a,b,j);
		s=(4.0*st-ost)/3.0; 

		printf("\t Simpson: %g \n", s);

		if (j > 5) 				// Avoid spurious early convergence.
			if (fabs(s-os) < EPS*fabs(os) ||
			   (s == 0.0 && os == 0.0)) return s;
		os=s;
		ost=st;
	}	
	nrerror("Too many steps in routine qsimp");
	return 0.0; 					// Never get here.
}

void main (int argc, char *argv[])
{

	int i;
	float a = 0.0, b = 3.1416, s = 0.0;

	for (i=1; i<argc; i++ ){
		
		switch(i){
	    	case 1: 
			a = atof(argv[1]); 
			break;
		case 2:
			b = atof(argv[2]); 
			break;
		} 
	}

	if    ( a < b ) s = qsimp( cos, a, b);
	else if ( b < a ) s = - qsimp( cos, b, a);
	
	printf("\n\n\n\t %g\n", b);
	printf("\t /\n");
	printf("\t | cos (x) dx = %.16f \t (Simpson)\n", s);
	printf("\t /\n");
	printf("\t %g\n", a);

}
